import { MutableRefObject, ReactNode } from 'react';
import { CheckboxOnChange, CheckboxState, ColumnProcessObj, FilterOnChange, PaginationOnChange, RowsPerPageOnChange, SortByPropOnChange, SortType } from '../helpers/types';
import { TableColumnType } from '../helpers/types';
/**
 * This is the additional parameters for the filter function.
 * Only applicable for uncontrolled table mode.
 */
export interface TableFilterParameters {
    /** The initial states for the table. */
    initialState?: {
        /** The initial text filter state. */
        filter: string;
    };
}
/**
 * This is the additional parameters for the filter function.
 * Only applicable for uncontrolled table mode.
 */
export interface TableSortParameters<TTableRowType> {
    /**
     * An object with the key being the table columns' prop and
     * the value being the value converter for the column.
     * This is most useful when we want to sort something by number
     * instead of by text.
     *
     * For example, we want to convert a date format
     * as the following: "Jan 22, 2022". Since string sorting will result
     * in a wrong result, then we need to convert it first, e.g. using `date-fns`.
     * After we parse the column's formatted date, only then we can get its
     * number value.
     *
     * ```ts
     * {
     *   sortValueObject: {
     *     date: (column: string) => parse(column).getTime()
     *   }
     * }
     * ```
     *
     * The object above will cause all rows in the `date` column to be sorted
     * by number (milliseconds) instead of by formatted date string.
     */
    sortValueObj?: ColumnProcessObj<TTableRowType, number>;
    /** The initial states for the table. */
    initialState?: SortType;
}
/**
 * This is the additional parameters for the filter function.
 * Only applicable for uncontrolled table mode.
 */
export interface TablePaginationParameters {
    /** The initial states for the table. */
    initialState?: {
        /** The initial currently active page. */
        page: number;
        /**
         * The initial maximum page. This is used to determine
         * the last numbered button in the pagination button group.
         * This also determines the next page number when the
         * "Last" button is clicked.
         */
        maxPage: number;
    };
}
/**
 * This is the additional parameters for the filter function.
 * Only applicable for uncontrolled table mode.
 */
export interface TablePaginationOptionsParameters {
    /** The initial states for the table. */
    initialState?: {
        /** Number of rows per page to be shown. */
        rowsPerPage: number;
        /** Rows per page options that will be shown in the dropdown. */
        options: number[];
    };
}
/**
 * This is the additional parameters for the filter function.
 * Only applicable for uncontrolled table mode.
 */
export interface TableCheckboxParameters {
    /** The initial states for the table. */
    initialState?: Record<string, CheckboxState>;
}
export interface UncontrolledTableEvents {
    onFilterChange: FilterOnChange;
    onSortByPropChange: SortByPropOnChange;
    onPaginationChange: PaginationOnChange;
    onRowsPerPageChange: RowsPerPageOnChange;
    onCheckboxChange: CheckboxOnChange;
}
/**
 * @internal
 *
 * This is the values stored in the `DatatableWrapper` context.
 */
interface DatatableWrapperContextType<TTableRowType> {
    headers: TableColumnType<TTableRowType>[];
    isFilterable: boolean;
    filterState: string;
    onFilterChange: (nextState: string) => void;
    sortState: SortType;
    onSortChange: (nextProp: SortType) => void;
    onSortByPropChange: (sortedProp: string) => void;
    currentPageState: number;
    onPaginationChange: (nextState: number) => void;
    rowsPerPageState: number;
    rowsPerPageOptions: number[];
    onRowsPerPageChange: (nextState: number) => void;
    checkboxState: Record<string, CheckboxState>;
    previouslyModifiedCheckbox: PreviouslyModifiedCheckbox;
    onCheckboxChange: CheckboxOnChange;
    checkboxRefs: MutableRefObject<Record<string, HTMLInputElement>>;
    maxPage: number;
    filteredDataLength: number;
    data: TTableRowType[];
    body: TTableRowType[];
}
/**
 * @internal
 *
 * This is an internal hook to use in all table components.
 */
export declare const useDatatableWrapper: () => DatatableWrapperContextType<any>;
/**
 * The props that can be passed to the `DatatableWrapper` component.
 */
export interface DatatableWrapperProps<TTableRowType> {
    /** The rest of the table, including its controls. */
    children: ReactNode;
    headers: TableColumnType<TTableRowType>[];
    body: TTableRowType[];
    /** When set to `true`, the table will "skip" all uncontrolled processes. */
    isControlled?: boolean;
    filterProps?: TableFilterParameters;
    sortProps?: TableSortParameters<TTableRowType>;
    paginationProps?: TablePaginationParameters;
    paginationOptionsProps?: TablePaginationOptionsParameters;
    checkboxProps?: TableCheckboxParameters;
    tableEventsRef?: MutableRefObject<UncontrolledTableEvents | undefined>;
}
/**
 * @internal
 *
 * This is an interface to represent the previously modified checkbox. This is used
 * for the "Select all" and "Deselect all" interaction.
 */
interface PreviouslyModifiedCheckbox {
    idProp: string;
    prop: string;
}
export declare function DatatableWrapper<TTableRowType = any>({ headers, body, isControlled, filterProps, sortProps, paginationProps, paginationOptionsProps, checkboxProps, tableEventsRef, children }: DatatableWrapperProps<TTableRowType>): JSX.Element;
export {};
